#include "TestSvgExport.h"

#include "../../electrical/ohmicresistance.h"
#include "../../electrical/twoport.h"
#include "../../export/svgExport.h"
#include "../../factory/factorybuilder.h"
#include "../../xmlparser/tinyxml2/xmlparserimpl.h"

// PreFunctions

template < typename T >
electrical::TwoPort< myMatrixType > *CreateElectricalElement()
{
    boost::shared_ptr< object::Object< double > > dummyObj;    // <--- deref will make boom
    return new T( dummyObj );
}

template < typename T >
electrical::TwoPortWithChild< myMatrixType > *CreateTwoPortWithChildren()
{
    return new T();
}


electrical::TwoPort< myMatrixType > *CreateComplexExample()
{
    electrical::TwoPortWithChild< myMatrixType > *mainSerialTwoPort(
     CreateTwoPortWithChildren< electrical::SerialTwoPort< myMatrixType > >() );

    electrical::TwoPortWithChild< myMatrixType > *twoPort(
     CreateTwoPortWithChildren< electrical::ParallelTwoPort< myMatrixType > >() );
    electrical::TwoPortWithChild< myMatrixType > *serialTwoPort(
     CreateTwoPortWithChildren< electrical::SerialTwoPort< myMatrixType > >() );
    electrical::TwoPortWithChild< myMatrixType > *parallelTwoPort(
     CreateTwoPortWithChildren< electrical::ParallelTwoPort< myMatrixType > >() );
    parallelTwoPort->AddChild( CreateElectricalElement< electrical::OhmicResistance< myMatrixType > >() );
    parallelTwoPort->AddChild( CreateElectricalElement< electrical::OhmicResistance< myMatrixType > >() );

    serialTwoPort->AddChild( CreateElectricalElement< electrical::OhmicResistance< myMatrixType > >() );
    serialTwoPort->AddChild( parallelTwoPort );
    serialTwoPort->AddChild( CreateElectricalElement< electrical::OhmicResistance< myMatrixType > >() );
    twoPort->AddChild( serialTwoPort );

    serialTwoPort = CreateTwoPortWithChildren< electrical::SerialTwoPort< myMatrixType > >();
    for ( size_t i = 0; i < 3; ++i )
        serialTwoPort->AddChild( CreateElectricalElement< electrical::OhmicResistance< myMatrixType > >() );
    twoPort->AddChild( serialTwoPort );
    mainSerialTwoPort->AddChild( twoPort );

    mainSerialTwoPort->AddChild( CreateElectricalElement< electrical::OhmicResistance< myMatrixType > >() );

    parallelTwoPort = CreateTwoPortWithChildren< electrical::ParallelTwoPort< myMatrixType > >();
    for ( size_t i = 0; i < 4; ++i )
        parallelTwoPort->AddChild( CreateElectricalElement< electrical::OhmicResistance< myMatrixType > >() );
    mainSerialTwoPort->AddChild( parallelTwoPort );

    return mainSerialTwoPort;
}

boost::shared_ptr< electrical::TwoPort< myMatrixType > > GenerateNetworkFromFile( const char *fileName )
{
    factory::FactoryBuilder< myMatrixType, double > factoryBuilder;
    factory::Factory< electrical::TwoPort< myMatrixType >, factory::ArgumentTypeElectrical > *electricalFactory =
     factoryBuilder.BuildElectricalFactory();

    boost::scoped_ptr< xmlparser::XmlParser > parser( new xmlparser::tinyxml2::XmlParserImpl() );
    parser->ReadFromFile( fileName );

    boost::shared_ptr< xmlparser::XmlParameter > rootParam( parser->GetRoot()->GetElementChild( "RootElement" ) );
    boost::shared_ptr< electrical::TwoPort< myMatrixType > > rootPort( electricalFactory->CreateInstance( rootParam ) );
    return rootPort;
}

// PreDefinitions

const char *correctPreamble =
 "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?> \n \
<svg \n \
    version=\"1.1\" \n \
    width=\"65\" \n \
    height=\"65\" \n \
    id=\"SVG_export ISEA\"> \n";

const char *ePerformancePack_Body =
 "<g\n"
 "     transform=\"translate(0, 812.5)\"\n"
 "     id=\"serialconnection\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 65,0\" \n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "  </g>\n"
 "<g\n"
 "     transform=\"translate(65, 260)\"\n"
 "     id=\"parallelconnection\">\n"
 "    <path\n"
 "       d=\"m 0,0 0,1267.5\" \n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
 "stroke-opacity:1;stroke-dasharray:none\" />\n"
 "  </g><g\n"
 "     transform=\"translate(65, 227.5)\"\n"
 "     id=\"serialconnection\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 65,0\" \n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "  </g>\n"
 "<g\n"
 "     transform=\"translate(130, 32.5)\"\n"
 "     id=\"parallelconnection\">\n"
 "    <path\n"
 "       d=\"m 0,0 0,455\" \n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
 "stroke-opacity:1;stroke-dasharray:none\" />\n"
 "  </g><g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 0)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 65)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 130)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 195)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 260)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 325)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 390)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 455)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     transform=\"translate(195, 32.5)\"\n"
 "     id=\"parallelconnection\">\n"
 "    <path\n"
 "       d=\"m 0,0 0,455\" \n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
 "stroke-opacity:1;stroke-dasharray:none\" />\n"
 "  </g><g\n"
 "     transform=\"translate(195, 227.5)\"\n"
 "     id=\"serialconnection\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 65,0\" \n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "  </g>\n"
 "<g\n"
 "     transform=\"translate(65, 715)\"\n"
 "     id=\"serialconnection\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 65,0\" \n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "  </g>\n"
 "<g\n"
 "     transform=\"translate(130, 552.5)\"\n"
 "     id=\"parallelconnection\">\n"
 "    <path\n"
 "       d=\"m 0,0 0,390\" \n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
 "stroke-opacity:1;stroke-dasharray:none\" />\n"
 "  </g><g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 520)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 585)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 650)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 715)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 780)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 845)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 910)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     transform=\"translate(195, 552.5)\"\n"
 "     id=\"parallelconnection\">\n"
 "    <path\n"
 "       d=\"m 0,0 0,390\" \n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
 "stroke-opacity:1;stroke-dasharray:none\" />\n"
 "  </g><g\n"
 "     transform=\"translate(195, 715)\"\n"
 "     id=\"serialconnection\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 65,0\" \n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "  </g>\n"
 "<g\n"
 "     transform=\"translate(65, 1137.5)\"\n"
 "     id=\"serialconnection\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 65,0\" \n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "  </g>\n"
 "<g\n"
 "     transform=\"translate(130, 1007.5)\"\n"
 "     id=\"parallelconnection\">\n"
 "    <path\n"
 "       d=\"m 0,0 0,325\" \n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
 "stroke-opacity:1;stroke-dasharray:none\" />\n"
 "  </g><g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 975)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 1040)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 1105)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 1170)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 1235)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 1300)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     transform=\"translate(195, 1007.5)\"\n"
 "     id=\"parallelconnection\">\n"
 "    <path\n"
 "       d=\"m 0,0 0,325\" \n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
 "stroke-opacity:1;stroke-dasharray:none\" />\n"
 "  </g><g\n"
 "     transform=\"translate(195, 1137.5)\"\n"
 "     id=\"serialconnection\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 65,0\" \n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "  </g>\n"
 "<g\n"
 "     transform=\"translate(65, 1495)\"\n"
 "     id=\"serialconnection\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 65,0\" \n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "  </g>\n"
 "<g\n"
 "     transform=\"translate(130, 1397.5)\"\n"
 "     id=\"parallelconnection\">\n"
 "    <path\n"
 "       d=\"m 0,0 0,260\" \n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
 "stroke-opacity:1;stroke-dasharray:none\" />\n"
 "  </g><g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 1365)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 1430)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 1495)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 1560)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     id=\"cellelement\"\n"
 "     transform=\"translate(130, 1625)\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\"\n"
 "       >c</text>\n"
 "  </g>\n"
 "<g\n"
 "     transform=\"translate(195, 1397.5)\"\n"
 "     id=\"parallelconnection\">\n"
 "    <path\n"
 "       d=\"m 0,0 0,260\" \n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
 "stroke-opacity:1;stroke-dasharray:none\" />\n"
 "  </g><g\n"
 "     transform=\"translate(195, 1495)\"\n"
 "     id=\"serialconnection\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 65,0\" \n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "  </g>\n"
 "<g\n"
 "     transform=\"translate(260, 260)\"\n"
 "     id=\"parallelconnection\">\n"
 "    <path\n"
 "       d=\"m 0,0 0,1267.5\" \n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
 "stroke-opacity:1;stroke-dasharray:none\" />\n"
 "  </g><g\n"
 "     transform=\"translate(260, 812.5)\"\n"
 "     id=\"serialconnection\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 65,0\" \n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "  </g>\n";

std::string tmpString =
 std::string(
  "<g\n"
  "     transform=\"translate(0, 227.5)\"\n"
  "     id=\"serialconnection\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 65,0\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(65, 130)\"\n"
  "     id=\"parallelconnection\">\n"
  "    <path\n"
  "       d=\"m 0,0 0,260\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-opacity:1;stroke-dasharray:none\" />\n"
  "  </g><g\n"
  "     transform=\"translate(65, 97.5)\"\n"
  "     id=\"serialconnection\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 65,0\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(130, 32.5)\"\n"
  "     id=\"parallelconnection\">\n"
  "    <path\n"
  "       d=\"m 0,0 0,195\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-opacity:1;stroke-dasharray:none\" />\n"
  "  </g><g\n"
  "     transform=\"translate(130, 0)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(195, 0)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(260, 0)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(130, 65)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(195, 65)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(260, 65)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(130, 130)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(195, 130)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(260, 130)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(130, 195)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(195, 195)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(260, 195)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(325, 32.5)\"\n"
  "     id=\"parallelconnection\">\n"
  "    <path\n"
  "       d=\"m 0,0 0,195\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-opacity:1;stroke-dasharray:none\" />\n"
  "  </g><g\n"
  "     transform=\"translate(325, 97.5)\"\n"
  "     id=\"serialconnection\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 65,0\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(390, 97.5)\"\n"
  "     id=\"serialconnection\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 65,0\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(455, 32.5)\"\n"
  "     id=\"parallelconnection\">\n"
  "    <path\n"
  "       d=\"m 0,0 0,195\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-opacity:1;stroke-dasharray:none\" />\n"
  "  </g><g\n"
  "     transform=\"translate(455, 0)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(520, 0)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(585, 0)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(455, 65)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(520, 65)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(585, 65)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(455, 130)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(520, 130)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(585, 130)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(455, 195)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(520, 195)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(585, 195)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(650, 32.5)\"\n"
  "     id=\"parallelconnection\">\n"
  "    <path\n"
  "       d=\"m 0,0 0,195\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-opacity:1;stroke-dasharray:none\" />\n"
  "  </g><g\n"
  "     transform=\"translate(650, 97.5)\"\n"
  "     id=\"serialconnection\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 65,0\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(715, 97.5)\"\n"
  "     id=\"serialconnection\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 65,0\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(780, 32.5)\"\n"
  "     id=\"parallelconnection\">\n"
  "    <path\n"
  "       d=\"m 0,0 0,195\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-opacity:1;stroke-dasharray:none\" />\n"
  "  </g><g\n"
  "     transform=\"translate(780, 0)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(845, 0)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(910, 0)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(780, 65)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(845, 65)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(910, 65)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(780, 130)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(845, 130)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(910, 130)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(780, 195)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(845, 195)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       " ) +
 std::string(
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(910, 195)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(975, 32.5)\"\n"
  "     id=\"parallelconnection\">\n"
  "    <path\n"
  "       d=\"m 0,0 0,195\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-opacity:1;stroke-dasharray:none\" />\n"
  "  </g><g\n"
  "     transform=\"translate(975, 97.5)\"\n"
  "     id=\"serialconnection\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 65,0\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(65, 357.5)\"\n"
  "     id=\"serialconnection\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 65,0\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(130, 292.5)\"\n"
  "     id=\"parallelconnection\">\n"
  "    <path\n"
  "       d=\"m 0,0 0,195\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-opacity:1;stroke-dasharray:none\" />\n"
  "  </g><g\n"
  "     transform=\"translate(130, 260)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(195, 260)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(260, 260)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(130, 325)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(195, 325)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(260, 325)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(130, 390)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(195, 390)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(260, 390)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(130, 455)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(195, 455)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(260, 455)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(325, 292.5)\"\n"
  "     id=\"parallelconnection\">\n"
  "    <path\n"
  "       d=\"m 0,0 0,195\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-opacity:1;stroke-dasharray:none\" />\n"
  "  </g><g\n"
  "     transform=\"translate(325, 357.5)\"\n"
  "     id=\"serialconnection\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 65,0\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(390, 357.5)\"\n"
  "     id=\"serialconnection\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 65,0\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(455, 292.5)\"\n"
  "     id=\"parallelconnection\">\n"
  "    <path\n"
  "       d=\"m 0,0 0,195\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-opacity:1;stroke-dasharray:none\" />\n"
  "  </g><g\n"
  "     transform=\"translate(455, 260)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(520, 260)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(585, 260)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(455, 325)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(520, 325)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(585, 325)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(455, 390)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(520, 390)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(585, 390)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(455, 455)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(520, 455)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(585, 455)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(650, 292.5)\"\n"
  "     id=\"parallelconnection\">\n"
  "    <path\n"
  "       d=\"m 0,0 0,195\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-opacity:1;stroke-dasharray:none\" />\n"
  "  </g><g\n"
  "     transform=\"translate(650, 357.5)\"\n"
  "     id=\"serialconnection\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 65,0\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(715, 357.5)\"\n"
  "     id=\"serialconnection\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 65,0\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(780, 292.5)\"\n"
  "     id=\"parallelconnection\">\n"
  "    <path\n"
  "       d=\"m 0,0 0,195\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-opacity:1;stroke-dasharray:none\" />\n"
  "  </g><g\n"
  "     transform=\"translate(780, 260)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(845, 260)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(910, 260)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(780, 325)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(845, 325)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(910, 325)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(780, 390)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       " ) +
 std::string(
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(845, 390)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(910, 390)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(780, 455)\"\n"
  "     id=\"ohmicresistance\">\n"
  "   <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(845, 455)\"\n"
  "     id=\"voltagesource\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 30,18.5 0,28\"\n"
  "       id=\"path21990\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 35,18.5 0,28\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "   <circle\n"
  "       cx=\"32.5\"\n"
  "       cy=\"32.5\"\n"
  "       r=\"17.5\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     id=\"zarc\"\n"
  "     transform=\"translate(910, 455)\">\n"
  "    <text\n"
  "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
  "       x=\"19.5\"\n"
  "       y=\"46\">z</text>\n"
  "    <path\n"
  "       d=\"m 0,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <path\n"
  "       d=\"m 50,32.5 15,0\"\n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "    <rect\n"
  "       width=\"35\"\n"
  "       height=\"15\"\n"
  "       x=\"15\"\n"
  "       y=\"25\"\n"
  "       "
  "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
  "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
  "inline;overflow:visible\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(975, 292.5)\"\n"
  "     id=\"parallelconnection\">\n"
  "    <path\n"
  "       d=\"m 0,0 0,195\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-opacity:1;stroke-dasharray:none\" />\n"
  "  </g><g\n"
  "     transform=\"translate(975, 357.5)\"\n"
  "     id=\"serialconnection\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 65,0\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "  </g>\n"
  "<g\n"
  "     transform=\"translate(1040, 130)\"\n"
  "     id=\"parallelconnection\">\n"
  "    <path\n"
  "       d=\"m 0,0 0,260\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:square;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-opacity:1;stroke-dasharray:none\" />\n"
  "  </g><g\n"
  "     transform=\"translate(1040, 227.5)\"\n"
  "     id=\"serialconnection\">\n"
  "    <path\n"
  "       d=\"m 0,32.5 65,0\" \n"
  "       "
  "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;"
  "stroke-"
  "opacity:1;stroke-dasharray:none\" />\n"
  "  </g>\n" );
const char *testShowBattery_Body = tmpString.c_str();
const char *nissanLeafSingleCell_Body =
 "<g\n"
 "     transform=\"translate(0, 0)\"\n"
 "     id=\"ohmicresistance\">\n"
 "   <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "  </g>\n"
 "<g\n"
 "     id=\"zarc\"\n"
 "     transform=\"translate(65, 0)\">\n"
 "    <text\n"
 "       style=\"font-size:50px;line-height:125%;fill:#000000;fill-opacity:1;font-family:sans\"\n"
 "       x=\"19.5\"\n"
 "       y=\"46\">z</text>\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"25\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "  </g>\n"
 "<g\n"
 "     id=\"parallelrc\"\n"
 "     transform=\"translate(130, 0)\">\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,14 8,0\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 50,14 8,0\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"7.5\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <path\n"
 "       d=\"m 0,32.5 7,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,32.5 7,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 27.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 37.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 38,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,12.5 0,39\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,12.5 0,39\" />\n"
 "  </g>\n"
 "<g\n"
 "     id=\"parallelrc\"\n"
 "     transform=\"translate(195, 0)\">\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,14 8,0\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 50,14 8,0\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"7.5\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <path\n"
 "       d=\"m 0,32.5 7,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,32.5 7,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 27.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 37.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 38,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,12.5 0,39\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,12.5 0,39\" />\n"
 "  </g>\n"
 "<g\n"
 "     id=\"parallelrc\"\n"
 "     transform=\"translate(260, 0)\">\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,14 8,0\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 50,14 8,0\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"7.5\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <path\n"
 "       d=\"m 0,32.5 7,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,32.5 7,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 27.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 37.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 38,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,12.5 0,39\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,12.5 0,39\" />\n"
 "  </g>\n"
 "<g\n"
 "     id=\"parallelrc\"\n"
 "     transform=\"translate(325, 0)\">\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,14 8,0\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 50,14 8,0\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"7.5\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <path\n"
 "       d=\"m 0,32.5 7,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,32.5 7,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 27.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 37.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 38,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,12.5 0,39\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,12.5 0,39\" />\n"
 "  </g>\n"
 "<g\n"
 "     id=\"parallelrc\"\n"
 "     transform=\"translate(390, 0)\">\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,14 8,0\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 50,14 8,0\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"7.5\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <path\n"
 "       d=\"m 0,32.5 7,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,32.5 7,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 27.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 37.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 38,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,12.5 0,39\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,12.5 0,39\" />\n"
 "  </g>\n"
 "<g\n"
 "     id=\"parallelrc\"\n"
 "     transform=\"translate(455, 0)\">\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,14 8,0\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 50,14 8,0\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"7.5\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <path\n"
 "       d=\"m 0,32.5 7,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,32.5 7,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 27.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 37.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 38,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,12.5 0,39\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,12.5 0,39\" />\n"
 "  </g>\n"
 "<g\n"
 "     id=\"parallelrc\"\n"
 "     transform=\"translate(520, 0)\">\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,14 8,0\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 50,14 8,0\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"7.5\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <path\n"
 "       d=\"m 0,32.5 7,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,32.5 7,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 27.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 37.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 38,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,12.5 0,39\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,12.5 0,39\" />\n"
 "  </g>\n"
 "<g\n"
 "     id=\"parallelrc\"\n"
 "     transform=\"translate(585, 0)\">\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,14 8,0\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 50,14 8,0\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"7.5\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <path\n"
 "       d=\"m 0,32.5 7,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,32.5 7,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 27.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 37.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 38,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,12.5 0,39\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,12.5 0,39\" />\n"
 "  </g>\n"
 "<g\n"
 "     id=\"parallelrc\"\n"
 "     transform=\"translate(650, 0)\">\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,14 8,0\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 50,14 8,0\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"7.5\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <path\n"
 "       d=\"m 0,32.5 7,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,32.5 7,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 27.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 37.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 38,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,12.5 0,39\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,12.5 0,39\" />\n"
 "  </g>\n"
 "<g\n"
 "     id=\"parallelrc\"\n"
 "     transform=\"translate(715, 0)\">\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,14 8,0\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 50,14 8,0\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"7.5\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <path\n"
 "       d=\"m 0,32.5 7,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,32.5 7,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 27.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 37.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 38,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,12.5 0,39\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,12.5 0,39\" />\n"
 "  </g>\n"
 "<g\n"
 "     id=\"parallelrc\"\n"
 "     transform=\"translate(780, 0)\">\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,14 8,0\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 50,14 8,0\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"7.5\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <path\n"
 "       d=\"m 0,32.5 7,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,32.5 7,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 27.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 37.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 38,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,12.5 0,39\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,12.5 0,39\" />\n"
 "  </g>\n"
 "<g\n"
 "     id=\"parallelrc\"\n"
 "     transform=\"translate(845, 0)\">\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,14 8,0\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 50,14 8,0\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"7.5\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <path\n"
 "       d=\"m 0,32.5 7,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,32.5 7,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 27.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 37.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 38,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,12.5 0,39\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,12.5 0,39\" />\n"
 "  </g>\n"
 "<g\n"
 "     id=\"parallelrc\"\n"
 "     transform=\"translate(910, 0)\">\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,14 8,0\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 50,14 8,0\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"7.5\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <path\n"
 "       d=\"m 0,32.5 7,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,32.5 7,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 27.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 37.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 38,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,12.5 0,39\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,12.5 0,39\" />\n"
 "  </g>\n"
 "<g\n"
 "     id=\"parallelrc\"\n"
 "     transform=\"translate(975, 0)\">\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,14 8,0\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 50,14 8,0\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"7.5\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <path\n"
 "       d=\"m 0,32.5 7,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,32.5 7,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 27.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 37.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 38,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,12.5 0,39\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,12.5 0,39\" />\n"
 "  </g>\n"
 "<g\n"
 "     id=\"parallelrc\"\n"
 "     transform=\"translate(1040, 0)\">\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,14 8,0\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 50,14 8,0\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"7.5\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <path\n"
 "       d=\"m 0,32.5 7,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,32.5 7,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 27.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 37.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 38,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,12.5 0,39\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,12.5 0,39\" />\n"
 "  </g>\n"
 "<g\n"
 "     id=\"parallelrc\"\n"
 "     transform=\"translate(1105, 0)\">\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,14 8,0\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 50,14 8,0\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"7.5\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <path\n"
 "       d=\"m 0,32.5 7,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,32.5 7,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 27.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 37.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 38,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,12.5 0,39\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,12.5 0,39\" />\n"
 "  </g>\n"
 "<g\n"
 "     id=\"parallelrc\"\n"
 "     transform=\"translate(1170, 0)\">\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,14 8,0\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 50,14 8,0\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"7.5\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <path\n"
 "       d=\"m 0,32.5 7,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,32.5 7,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 27.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 37.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 38,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,12.5 0,39\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,12.5 0,39\" />\n"
 "  </g>\n"
 "<g\n"
 "     id=\"parallelrc\"\n"
 "     transform=\"translate(1235, 0)\">\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,14 8,0\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 50,14 8,0\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"7.5\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <path\n"
 "       d=\"m 0,32.5 7,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,32.5 7,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 27.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 37.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 38,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,12.5 0,39\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,12.5 0,39\" />\n"
 "  </g>\n"
 "<g\n"
 "     id=\"parallelrc\"\n"
 "     transform=\"translate(1300, 0)\">\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,14 8,0\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 50,14 8,0\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"7.5\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <path\n"
 "       d=\"m 0,32.5 7,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,32.5 7,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 27.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 37.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 38,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,12.5 0,39\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,12.5 0,39\" />\n"
 "  </g>\n"
 "<g\n"
 "     id=\"parallelrc\"\n"
 "     transform=\"translate(1365, 0)\">\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,14 8,0\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 50,14 8,0\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"7.5\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <path\n"
 "       d=\"m 0,32.5 7,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,32.5 7,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 27.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 37.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 38,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,12.5 0,39\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,12.5 0,39\" />\n"
 "  </g>\n"
 "<g\n"
 "     id=\"parallelrc\"\n"
 "     transform=\"translate(1430, 0)\">\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,14 8,0\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 50,14 8,0\" />\n"
 "    <rect\n"
 "       width=\"35\"\n"
 "       height=\"15\"\n"
 "       x=\"15\"\n"
 "       y=\"7.5\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "    <path\n"
 "       d=\"m 0,32.5 7,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,32.5 7,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 27.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 37.5,40 0,20\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 38,50 20,0\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 7,12.5 0,39\" />\n"
 "   <path\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\"\n"
 "       d=\"m 58,12.5 0,39\" />\n"
 "  </g>\n"
 "<g\n"
 "     transform=\"translate(1495, 0)\"\n"
 "     id=\"voltagesource\">\n"
 "    <path\n"
 "       d=\"m 0,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 50,32.5 15,0\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 30,18.5 0,28\"\n"
 "       id=\"path21990\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "    <path\n"
 "       d=\"m 35,18.5 0,28\"\n"
 "       "
 "style=\"fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:miter;stroke-miterlimit:4;stroke-"
 "opacity:1;stroke-dasharray:none\" />\n"
 "   <circle\n"
 "       cx=\"32.5\"\n"
 "       cy=\"32.5\"\n"
 "       r=\"17.5\"\n"
 "       "
 "style=\"color:#000000;fill:none;stroke:#000000;stroke-width:3;stroke-linecap:butt;stroke-linejoin:round;stroke-"
 "miterlimit:4;stroke-opacity:1;stroke-dasharray:none;stroke-dashoffset:0;marker:none;visibility:visible;display:"
 "inline;overflow:visible\" />\n"
 "  </g>\n";

const char *correctEnding = "</svg>\n";

// TestFunctions

TestSvgExport::TestSvgExport()
    : mDummyOutstream( "" )
    , mTestDummy( CreateTwoPortWithChildren< electrical::ParallelTwoPort< myMatrixType > >() )
{
}

void TestSvgExport::testSvgExportFailTwoPort()
{
    TS_ASSERT_THROWS_ASSERT( SvgExport<> testObject( 0, 0 ), const std::runtime_error &e,
                             TS_ASSERT_EQUALS( strcmp( e.what(), "No rootport specified.\n" ), 0 ) );
}

void TestSvgExport::testSvgExportFailStream()
{
    boost::scoped_ptr< electrical::TwoPort< myMatrixType > > twoPort(
     CreateElectricalElement< electrical::OhmicResistance< myMatrixType > >() );

    TS_ASSERT_THROWS_ASSERT( SvgExport<> testObject( twoPort.get(), 0 ), const std::runtime_error &e,
                             TS_ASSERT_EQUALS( strcmp( e.what(), "No stream defined in SVGexport specified.\n" ), 0 ) );
}

void TestSvgExport::testSvgExportStopEvaluateDepth()
{
    boost::scoped_ptr< electrical::TwoPortWithChild< myMatrixType > > twoPort(
     CreateTwoPortWithChildren< electrical::ParallelTwoPort< myMatrixType > >() );

    std::vector< std::string > twoPortNames{"OhmicResistance",      "VoltageSoruce", "Capacity",
                                            "ParallelRC",           "Zarc",          "Inductance",
                                            "ConstantPhaseElement", "WarburgTanh",   "WarburgCotanh",
                                            "Cellelement",          "SerialTwoport", "ParallelTwoPort"};

    std::vector< size_t > celllevelDepth{0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0};
    std::vector< size_t > electricalelementsDepth{0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0};
    std::vector< size_t > all{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

    TS_ASSERT_EQUALS( twoPortNames.size(), celllevelDepth.size() );
    TS_ASSERT_EQUALS( twoPortNames.size(), electricalelementsDepth.size() );
    TS_ASSERT_EQUALS( twoPortNames.size(), all.size() );

    SvgExport< myMatrixType, CELLLEVEL_DEPTH > testObjectCelllevelDepth( mTestDummy.get(), &mDummyOutstream );
    SvgExport< myMatrixType, ELECTRICALELEMENTS_DEPTH > testObjectElectricalElementsDepth( mTestDummy.get(), &mDummyOutstream );
    SvgExport< myMatrixType, ALL > testObjectAll( mTestDummy.get(), &mDummyOutstream );

    for ( size_t i = 0; i < twoPortNames.size(); ++i )
    {
        std::string name = twoPortNames[i];

        TS_ASSERT_EQUALS( testObjectCelllevelDepth.StopEvaluateDepth( name.c_str() ), celllevelDepth[i] );
        TS_ASSERT_EQUALS( testObjectElectricalElementsDepth.StopEvaluateDepth( name.c_str() ), electricalelementsDepth[i] );
        TS_ASSERT_EQUALS( testObjectAll.StopEvaluateDepth( name.c_str() ), all[i] );
    }
}

void TestSvgExport::testSvgExportGetHeightOfTwoPort()
{
    boost::shared_ptr< electrical::TwoPortWithChild< myMatrixType > > oneElementHeightTwoPort(
     CreateTwoPortWithChildren< electrical::SerialTwoPort< myMatrixType > >() );
    boost::shared_ptr< electrical::TwoPort< myMatrixType > > fourElementsHeightTwoPort( CreateComplexExample() );

    for ( size_t i = 0; i < 7; ++i )
    {
        oneElementHeightTwoPort->AddChild( CreateElectricalElement< electrical::OhmicResistance< myMatrixType > >() );
    }

    SvgExport<> testSvgExport( mTestDummy.get(), &mDummyOutstream );

    TS_ASSERT_EQUALS( testSvgExport.GetHeightOfTwoPort( oneElementHeightTwoPort.get() ), 1 );

    TS_ASSERT_EQUALS( testSvgExport.GetHeightOfTwoPort( fourElementsHeightTwoPort.get() ), 4 );
}

void TestSvgExport::testPreamble()
{
    std::stringstream testPreamble_Stream;

    SvgExport<> testSvgExport( mTestDummy.get(), &testPreamble_Stream );
    testPreamble_Stream.str( "" );

    testSvgExport.mMaxHeight = 1;
    testSvgExport.mMaxWidth = 1;

    testSvgExport.WritePreamble();

    TS_ASSERT_EQUALS( std::string( correctPreamble ), testPreamble_Stream.str() );
}

void TestSvgExport::testEnding()
{
    std::stringstream ss;

    SvgExport<> testSvgExport( mTestDummy.get(), &ss );
    ss.str( "" );

    testSvgExport.WriteEnding();

    TS_ASSERT_EQUALS( correctEnding, ss.str() );
}
